/*	ArgusGUI utility functions
	GetFolder
	GetFile
*/

#include "stdafx.h"
#include "winnetwk.h"
#include "shlobj.h"
#include "resource.h"
#include "argus_api.h"
#include "ArgusGUI.h"
#include "ArgusGUIProgressDialog.h"
#include <assert.h>

int __stdcall DirectorySelectCallBack(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData);

BOOL GetFolder(HWND hwndOwner, CWnd* ctrl, UINT nID, CString& dirEdit)
{
	BROWSEINFO info;
	LPITEMIDLIST lpitem; 
	LPMALLOC g_pMalloc; 
	CString title;
	CString path = dirEdit;

	if(path.ReverseFind('\\')==path.GetLength())	
		path = path.Left(path.GetLength() - 1); 

	title.LoadString(nID);

	SHGetMalloc(&g_pMalloc);

	char folder[MAX_PATH]; // Used for storing the folders pathname

	//Initialize the BROWSEINFO structure
	info.hwndOwner=hwndOwner;
	info.pidlRoot=NULL;
	info.pszDisplayName=folder; //Store selected item here
	info.lpszTitle=title; //Display this text
	info.ulFlags=BIF_RETURNONLYFSDIRS; //Only directories are valid selctions
	info.iImage=0;
	if(path != "")
	{
		info.lpfn=DirectorySelectCallBack;
		info.lParam = (LPARAM)(const char*)path;
	}
	else
	{
		info.lpfn=NULL;
		info.lParam = (LPARAM)NULL;
	}


	// Get the ITEMIDLIST object
	// The return ptr must be freed manually, see later
	if((lpitem=SHBrowseForFolder(&info))==NULL)
		return FALSE; 

	// Convert the ITEMIDLIST to the complete folder name
	if(!SHGetPathFromIDList(lpitem,folder))
	{
		AfxMessageBox(IDS_UNKNOWN_ERROR);
		return FALSE;
	}

	// Free the lpitem object
	g_pMalloc->Free(lpitem); 

	// Copy the folder to the CEdit control
	// Make sure the last character is an backslash
	dirEdit=folder;
	if(ctrl != NULL)
		ctrl->SetWindowText(folder);

	return TRUE;
}

int __stdcall DirectorySelectCallBack(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	if(uMsg == BFFM_INITIALIZED)
	{
		SendMessage(hwnd, BFFM_SETSELECTION, TRUE, lpData);
	}

	return 0;
}

void displayError(int code, char* errorStr)
{
	CString message, caption, format;

	format.LoadString(IDS_PROCESSINGDOCERROR);
	message.Format(format, code, errorStr);

	caption.LoadString(IDS_ARGUSGUICAPTION);

	MessageBox(AfxGetMainWnd()->m_hWnd, message, caption, MB_OK | MB_ICONEXCLAMATION);

}

void guiMessage(void* guiData, int level, char* text)
{
	CArgusGUIProgressDialog* progress = (CArgusGUIProgressDialog*)guiData;

	progress->displayMessage(level, text);
}

void guiGauge(void* guiData, int percent)
{
	CArgusGUIProgressDialog* progress = (CArgusGUIProgressDialog*)guiData;

	progress->displayGauge(percent);
}

int guiCancel(void* guiData)
{
	int x = 4;

	return (FALSE);
}
