// CropDialog.cpp : implementation file
//

#include "stdafx.h"
#include "argus_api.h"
#include "argusgui.h"
#include "CropDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static int endPage(void* data);
static int fillPoly(IPPoint* pts, int nofPoints, ICNColour* rgb, void* data);
static int drawPoly(IPPoint* pts, int nofPoints, ICNColour* rgb, void* data);
static int showImage(IPQuad* quad, IPVImage vimage, int w, int h, int bpc, int nofChannels, void* data);
static int showText(IPQuad* quad, const ichar* text, int nofBytes, u8* fname, void* data);
static int beginPage(IPRect* bbox, IPRect* crop, void* data);
static void transform(CCropDialog* crop, long* x, long* y, float fx, float fy);

/* --------------------------------------------------------- */
CCropDialog::CCropDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CCropDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCropDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	metaHandle = NULL;
}

BOOL CCropDialog::DestroyWindow() 
{
	/* Work out how to scale the users bbox from pixels back into PDF space */
	CStatic* frame = (CStatic*)GetDlgItem(IDC_FRAME);
	float pageWidth = pageRect.right - pageRect.left;
	float pageHeight = pageRect.top - pageRect.bottom;
	RECT frameRect;
	float xScale;
	float yScale;

	frame->GetClientRect(&frameRect);

	xScale = pageWidth / frameRect.right;
	yScale = pageHeight / frameRect.bottom;
	
	/* Now work out the cropping rectangle in PDF space */
	theApp.cropBox.left = bbox.left * xScale;
	theApp.cropBox.right = bbox.right * xScale;

	/* Remember to invert y coords again (bit confusing realy) */
	theApp.cropBox.top = bbox.top * yScale;
	theApp.cropBox.bottom = (frameRect.bottom - bbox.bottom) * yScale;

	/* Destroy metaFile */
	if (metaHandle != NULL)
	{
		DeleteEnhMetaFile(metaHandle);
		metaHandle = NULL;
	}	
	/* Free the rendering instance */
	icnArgusRenderDestroy(render);

	return CDialog::DestroyWindow();
}

void CCropDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCropDialog)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCropDialog, CDialog)
	//{{AFX_MSG_MAP(CCropDialog)
	ON_WM_PAINT()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCropDialog message handlers

BOOL CCropDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	CSpinButtonCtrl* spin = (CSpinButtonCtrl*)GetDlgItem(IDC_PAGE_SPIN);
	ICNArgusRenderCallbacks mon;

	/* Set up the rendering callbacks structure */
	mon.beginPage = beginPage;
	mon.showText = showText;
	mon.paintImage = showImage;
	mon.drawPoly = drawPoly;
	mon.fillPoly = fillPoly;
	mon.endPage = endPage;
	mon.dataBlock = this;

	/* Create a new rendering instance for this document */
	render = icnArgusRenderCreate(theApp.cropFileName, &mon);
	if (render == NULL)
		return FALSE;
	
	/* Get num pages to setup spin button */
	nofPages = icnArgusRenderGetPageCount(render);

	/* Set up the page spin button control */
	spin->SetRange(1, nofPages);
	spin->SetPos(1);
	pageNum = 0;
	
	/* Set the title of the window */
	SetWindowText(theApp.cropFileName);
	return TRUE;
}

void CCropDialog::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	/* On first time thru, setup up spin buttons and first page sketch */
	if (metaHandle == NULL)
	{
		CSpinButtonCtrl* leftSpin = (CSpinButtonCtrl*)GetDlgItem(IDC_LEFT_SPIN);
		CSpinButtonCtrl* rightSpin = (CSpinButtonCtrl*)GetDlgItem(IDC_RIGHT_SPIN);
		CSpinButtonCtrl* topSpin = (CSpinButtonCtrl*)GetDlgItem(IDC_TOP_SPIN);
		CSpinButtonCtrl* bottomSpin = (CSpinButtonCtrl*)GetDlgItem(IDC_BOTTOM_SPIN);
		CStatic* frame = (CStatic*)GetDlgItem(IDC_FRAME);
	
		renderPageImage();

		/* Set up initial crop box */
		frame->GetClientRect(&bbox);

		/* Set up crop box spin buttons */
		leftSpin->SetRange(0,bbox.right-1);
		leftSpin->SetPos(0);

		rightSpin->SetRange(0,bbox.right-1);
		rightSpin->SetPos(bbox.right-1);
		
		/* Initial values for these may look screwy but its because Win Y coords
			are upside down w.r.t. PDF coords. */
		topSpin->SetRange(0,bbox.bottom-1);
		topSpin->SetPos(bbox.bottom-1);
		
		bottomSpin->SetRange(0,bbox.bottom-1);
		bottomSpin->SetPos(0);
	}
	drawFrame();
}

void CCropDialog::drawFrame(void)
{
	/* Draw the meta file */
	CStatic* frame = (CStatic*)GetDlgItem(IDC_FRAME);
	RECT frameBox;
	CDC* fdc;

	frame->GetClientRect(&frameBox);
	fdc = frame->GetDC();
	fdc->PlayMetaFile(metaHandle, &frameBox);
	
	/* Draw the crop box */
	drawCropBox();
	frame->ReleaseDC(fdc);
}

void CCropDialog::drawCropBox(void)
{
	/* Draw simple cropping lines on the frame */
	CStatic* frame = (CStatic*)GetDlgItem(IDC_FRAME);
	CDC*cdc = frame->GetDC();
	RECT frameRect;
	long height;
	int oldMode;

	frame->GetClientRect(&frameRect);
	height = frameRect.bottom;

	cdc->BeginPath();
	/* top line */
	cdc->MoveTo(0, height - bbox.top);
	cdc->LineTo(frameRect.right, height - bbox.top);

	/* bottom line */
	cdc->MoveTo(0, height - bbox.bottom);
	cdc->LineTo(frameRect.right, height - bbox.bottom);

	/* Left line */
	cdc->MoveTo(bbox.left, 0);
	cdc->LineTo(bbox.left, frameRect.bottom);

	/* Right line */
	cdc->MoveTo(bbox.right, 0);
	cdc->LineTo(bbox.right, frameRect.bottom);
	cdc->EndPath();
	
	oldMode = cdc->SetROP2(R2_NOT);
	cdc->StrokePath();
	cdc->SetROP2(oldMode);
	frame->ReleaseDC(cdc);
}

void CCropDialog::renderPageImage(void)
{
	/* Render an image of the page into the given cdc. Filename of page is stored
		in the app.
	*/
	CStatic* frame = (CStatic*)GetDlgItem(IDC_FRAME);

	/* Delete old metafile */
	if (metaHandle != NULL)
		DeleteEnhMetaFile(metaHandle);

	/* Create a new metafile */
	hdc = CreateEnhMetaFile(NULL, NULL, NULL, NULL); 
	icnArgusRenderPage(render, pageNum);

	metaHandle = CloseEnhMetaFile(hdc);
	hdc = NULL;
}

void CCropDialog::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	// TODO: Add your message handler code here and/or call default
	CSpinButtonCtrl* spin = (CSpinButtonCtrl*)pScrollBar;
	CSpinButtonCtrl* leftSpin = (CSpinButtonCtrl*)GetDlgItem(IDC_LEFT_SPIN);
	CSpinButtonCtrl* rightSpin = (CSpinButtonCtrl*)GetDlgItem(IDC_RIGHT_SPIN);
	long* target;

	if (spin == leftSpin)
	{
		target = &bbox.left; 		/* Move left crop line */
	}
	else if (spin == rightSpin)
	{
		target = &bbox.right;		/* Move right crop line */
	}
	else /* Page left/right spin */
	{
		CStatic* frame = (CStatic*)GetDlgItem(IDC_FRAME);
		int newPage = spin->GetPos() - 1;

		if (newPage != pageNum)
		{
			pageNum = newPage;
			renderPageImage();
			drawFrame();
		}
		return;
	}
	
	/* Erase, move then redraw crop box */
	drawCropBox();
	*target = spin->GetPos();
	drawCropBox();

	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CCropDialog::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CSpinButtonCtrl* spin = (CSpinButtonCtrl*)pScrollBar;
	CSpinButtonCtrl* topSpin = (CSpinButtonCtrl*)GetDlgItem(IDC_TOP_SPIN);
	CSpinButtonCtrl* bottomSpin = (CSpinButtonCtrl*)GetDlgItem(IDC_BOTTOM_SPIN);
	long* target;

	if (spin == topSpin)
	{
		target = &bbox.top;
	}
	else if (spin == bottomSpin)
	{
		target = &bbox.bottom;
	}
	
	drawCropBox();
	*target = spin->GetPos();
	drawCropBox();
	
	CDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

/* ------------------------------------------------------------------ */
static int beginPage(IPRect* mediaBox, IPRect* cropBox, void* data)
{
	CCropDialog* crop = (CCropDialog*)data;
	RECT rect;

	crop->pageRect = *cropBox;

	transform(crop, &rect.left, &rect.top, crop->pageRect.left, crop->pageRect.top);	
	transform(crop, &rect.right, &rect.bottom, crop->pageRect.right, crop->pageRect.bottom);

	FillRect(crop->hdc, &rect, (HBRUSH)GetStockObject(WHITE_BRUSH));

	return(TRUE);
}

static int showText(IPQuad* quad, const ichar* text, int nofBytes, u8* fname, void* data)
{
	HBRUSH brush = CreateSolidBrush(RGB(127,127,127));
	CCropDialog* crop = (CCropDialog*)data;
	HDC hdc = crop->hdc;
	HBRUSH oldBrush;
	POINT pts[4];

	/* Do nothing if it's just a space on its own */
	if (text[0] == ' ' && text[1] == '\0')
			return(TRUE);

	transform(crop, &pts[0].x, &pts[0].y, quad->tl.h, quad->tl.v);
	transform(crop, &pts[1].x, &pts[1].y, quad->tr.h, quad->tr.v);
	transform(crop, &pts[2].x, &pts[2].y, quad->br.h, quad->br.v);
	transform(crop, &pts[3].x, &pts[3].y, quad->bl.h, quad->bl.v);

	BeginPath(hdc);
	Polygon(hdc, pts, 4);
	EndPath(hdc);

	oldBrush = (HBRUSH)SelectObject(hdc, brush);
	FillPath(hdc);
	SelectObject(hdc, oldBrush);
	DeleteObject(brush);
	
	return(TRUE);
}

static int showImage(IPQuad* quad, IPVImage vimage, int w, int h, int bpc, int nofChannels, void* data)
{
	CCropDialog* crop = (CCropDialog*)data;
	HDC hdc = crop->hdc;
	IPRect bbox;
	RECT rect;

	/* Work out the bounding box of the image quad (in most cases the same as the quad) */
	bbox.top = max(quad->tl.v, quad->tr.v);
	bbox.bottom = min(quad->bl.v, quad->br.v);
	bbox.left = min(quad->tl.h, quad->bl.h);
	bbox.right = max(quad->tr.h, quad->br.h);
	
	transform(crop, &rect.left, &rect.top, bbox.left, bbox.top);
	transform(crop, &rect.right, &rect.bottom, bbox.right, bbox.bottom);

	Rectangle(hdc, rect.left, rect.top, rect.right, rect.bottom);
	return(TRUE);
}

static int drawPoly(IPPoint* pts, int nofPoints, ICNColour* rgb, void* data)
{
	unsigned long col = RGB(rgb->red*255.0, rgb->green*255.0, rgb->blue*255.0);
	CPen* pen = new CPen(PS_SOLID, 1, col);
	CCropDialog* crop = (CCropDialog*)data;
	HDC hdc = crop->hdc;
	CPen* oldPen;
	POINT oldPoint;
	long x;
	long y;
	int i;

	BeginPath(hdc);

	transform(crop, &x, &y, pts[0].h, pts[0].v);
	MoveToEx(hdc, x,y, &oldPoint);

	for(i=1; i < nofPoints; i++)
	{
		transform(crop, &x, &y, pts[i].h, pts[i].v);
		LineTo(hdc, x,y);
	}
	EndPath(hdc);

	oldPen = (CPen*)SelectObject(hdc, pen);	
	StrokePath(hdc);

	SelectObject(hdc, oldPen);
	pen->DeleteObject();
	delete(pen);
	return(TRUE);
}

static int fillPoly(IPPoint* pts, int nofPoints, ICNColour* rgb, void* data)
{
	unsigned long col = RGB(rgb->red*255.0, rgb->green*255.0, rgb->blue*255.0);
	HBRUSH brush = CreateSolidBrush(col);
	CCropDialog* crop = (CCropDialog*)data;
	HDC hdc = crop->hdc;
	HBRUSH oldBrush;
	POINT oldPoint;
	long x;
	long y;
	int i;

	BeginPath(hdc);

	transform(crop, &x, &y, pts[0].h, pts[0].v);
	MoveToEx(hdc, x,y, &oldPoint);

	for(i=1; i < nofPoints; i++)
	{
		transform(crop, &x, &y, pts[i].h, pts[i].v);
		LineTo(hdc, x,y);
	}
	EndPath(hdc);

	oldBrush = (HBRUSH)SelectObject(hdc, brush);
	FillPath(hdc);
	SelectObject(hdc, oldBrush);
	DeleteObject(brush);
	return(TRUE);
}

static int endPage(void* data)
{
	return(TRUE);
}

static void transform(CCropDialog* crop, long* x, long* y, float fx, float fy)
{
	if (fx < crop->pageRect.left)
			fx = crop->pageRect.left;
	else if (fx > crop->pageRect.right)
			fx = crop->pageRect.right;

	*x = (long)(fx + 0.5);

	/* Also invert the y direction since Argus 0,0 is bottom left & Win's is top left */
	if (fy > crop->pageRect.top)
			fy = crop->pageRect.top;
	else if (fy < crop->pageRect.bottom)
			fy = crop->pageRect.bottom;
	
	*y = (long)(crop->pageRect.bottom - fy + 0.5);
}

