Option Strict Off
Option Explicit On
Imports VB = Microsoft.VisualBasic
Friend Class VBArgus
	Inherits System.Windows.Forms.Form
	Private Sub aboutButton_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles aboutButton.Click
		VB6.ShowForm(frmAbout, (VB6.FormShowConstants.Modal))
	End Sub
	
	Private Sub configBrowse_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles configBrowse.Click
		Dim sFile As String
		
        With CommonDialogOpen
            .Title = "Choose a configuration file"
            .Filter = "Config Files (*.cfg)|*.cfg|All Files (*.*)|*.*"
            .ShowDialog()
            If Len(.FileName) = 0 Then
                Exit Sub
            End If
            sFile = .FileName
        End With
		
		' Store the chosen file in the associated text box
		configBox.Text = sFile
	End Sub
	
	Private Sub configBrowseButton_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles configBrowseButton.Click
        FolderBrowserDialog1.ShowDialog()
        outputTextBox.Text = FolderBrowserDialog1.SelectedPath
    End Sub
	
	Private Sub memoryProcess_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles memoryProcess.Click
		Dim numpages As Object
		Dim doc As Object
		' This is where everything happens...
		Dim vargus As Integer
        Dim error_Renamed As Short
		Dim startPage As Short
		Dim endPage As Short
		Dim pageNum As Short
		Dim pageText As String
		
		' Start the engine with the chosen config file
		vargus = icnArgusCreate()
		If vargus = 0 Then
			showArgusError(("Starting Engine"))
			Exit Sub
		End If
		
		' Tell the pdf parser about the password (if any)
		ipSetUserPassword((Password.Text))
		
        ' Load the test configuration file to tell argus what format of output to produce
		error_Renamed = icnArgusLoadConfig(vargus, Me.configBox.Text)
		If (error_Renamed <> 0) Then
			showArgusError(("Loading Config '" & Me.configBox.Text & "'"))
			icnArgusDestroy((vargus))
			Exit Sub
		End If
		
        ' Add progress callbacks - SDK will call these functions during processing
        Dim cbCancel As New ArgusSDK.ICNProgCancel(AddressOf progCancel)
        Dim cbMsg As New ArgusSDK.ICNProgMsg(AddressOf progMsg)
        Dim cbGauge As New ArgusSDK.ICNProgGauge(AddressOf progGauge)

        Call icnArgusSetProgress(vargus, cbMsg, cbGauge, cbCancel)

        ' Show the progress fuel gauge and make it small
        Me.fuelGauge.Width = 0
        Me.fuelGauge.Visible = True

        ' UNless you do this the callbacks may be garbage collected during processing
        ' within the DLL.
        GC.KeepAlive(cbCancel)
        GC.KeepAlive(cbMsg)
        GC.KeepAlive(cbGauge)

		' Open the chosen document
        doc = icnArgusDocCreate(vargus, pdfTextBox.Text)
        If doc = 0 Then
            showArgusError(("Loading document '" & pdfTextBox.Text & "'"))
            icnArgusDestroy((vargus))
            Exit Sub
        End If
		
		' Work out the start and end pages
		startPage = CShort(fromPageTextBox.Text) - 1
		endPage = CShort(toPageTextBox.Text) - 1
		
		' Get nof pages in document
        numpages = icnDocGetPageCount(doc)
        If endPage > numpages Then
            endPage = numpages - 1
        End If
		
		' Analyse just the pages of the document you want to extract
        error_Renamed = icnArgusDocAnalyse(vargus, doc, startPage, endPage)
		If (error_Renamed <> 0) Then
            showArgusError("Analysing Pages")
            icnDocDestroy(doc)
            icnArgusDestroy(vargus)
			Exit Sub
		End If
		
		' Get the text from each page in turn
		For pageNum = startPage To endPage
            memoryForm.pageNumBox.Text = CStr(pageNum + 1)

            pageText = icnArgusDocGetText(vargus, doc, pageNum)
            memoryForm.pageContents.Text = pageText

            ' Get the dll to free the memory allocated for the text
            icnDestroyText(pageText)

            VB6.ShowForm(memoryForm, (VB6.FormShowConstants.Modal))

			' Did the user press cancel?
			If memoryForm.pleaseStop = 1 Then
				Exit For
			End If
		Next 
		
		' Shut down the engine
		icnDocDestroy((doc))
		icnArgusDestroy((vargus))
		VersionMessage.Text = "Complete"
		
		' Hide the fuel gauge again
		Me.fuelGauge.Visible = False
	End Sub
	
	Private Sub pdfBrowseButton_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles pdfBrowseButton.Click
		Dim sFile As String
		
        With CommonDialogOpen
            .Title = "Choose a PDF to process"
            .Filter = "PDF Files (*.pdf)|*.pdf|All Files (*.*)|*.*"
            .ShowDialog()
            If Len(.FileName) = 0 Then
                Exit Sub
            End If
            sFile = .FileName
        End With
		
		' Store the chosen file in the associated text box
		pdfTextBox.Text = sFile
	End Sub
	Private Sub processButton_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles processButton.Click
		' This is where everything happens...
		Dim vargus As Integer
        Dim error_Renamed As Short
		Dim statMsg As New VB6.FixedLengthString(255)
		
		' Start the engine with the chosen config file
		vargus = icnArgusCreate()
		If vargus = 0 Then
			showArgusError(("Starting Engine"))
			Exit Sub
		End If
		
		' Tell the pdf parser about the password (if any)
		ipSetUserPassword((Password.Text))
		
		' Load the test configuration file to tell argus to produce HTML output
		error_Renamed = icnArgusLoadConfig(vargus, Me.configBox.Text)
		If (error_Renamed <> 0) Then
			showArgusError(("Loading Config '" & Me.configBox.Text & "'"))
			Exit Sub
		End If
		
		' Add a VALUE name called "PATH" which is the output folder
		' chosen by the user
		error_Renamed = icnArgusValueAdd(vargus, "outputFolder", outputTextBox.Text)
		If (error_Renamed <> 0) Then
			showArgusError(("adding PATH macro"))
			icnArgusDestroy((vargus))
			Exit Sub
		End If
		
		' Add progress callbacks - SDK will call these functions during processing
        Dim cbCancel As New ArgusSDK.ICNProgCancel(AddressOf progCancel)
        Dim cbMsg As New ArgusSDK.ICNProgMsg(AddressOf progMsg)
        Dim cbGauge As New ArgusSDK.ICNProgGauge(AddressOf progGauge)

        Call icnArgusSetProgress(vargus, cbMsg, cbGauge, cbCancel)
		
		' Show the progress fuel gauge and make it small
		Me.fuelGauge.Width = 0
		Me.fuelGauge.Visible = True
		
        GC.KeepAlive(cbCancel)
        GC.KeepAlive(cbMsg)
        GC.KeepAlive(cbGauge)

        ' Process the chosen PDF document
        error_Renamed = icnArgusExtract(vargus, pdfTextBox.Text, CShort(fromPageTextBox.Text) - 1, CShort(toPageTextBox.Text) - 1)
		If error_Renamed <> 0 Then
			showArgusError(("Processing Document"))
			Exit Sub
		End If
		
		' Shut down the engine
		icnArgusDestroy((vargus))
		VersionMessage.Text = "Complete"
		
		' Hide the fuel gauge again
		Me.fuelGauge.Visible = False
	End Sub
	Public Sub showArgusError(ByVal errorTitle As String)
		Dim errorMsg As New VB6.FixedLengthString(255)
		Dim stdMsg As New VB6.FixedLengthString(255)
		Dim nofBytes As Short
		Dim nofStdBytes As Short
		
		' Get the details of the error message
		nofBytes = icnErrorGetText(errorMsg.Value, 255)
		
		' Get the standard part of error message
		nofStdBytes = icnErrorGetStdText(stdMsg.Value, 255)
		
		' Display them both in the alert window
		AlertForm.Text = "Error " & errorTitle
		AlertForm.Label1.Text = VB.Left(stdMsg.Value, nofStdBytes) & ", " & VB.Left(errorMsg.Value, nofBytes)
		AlertForm.Show()
	End Sub
	
	Private Sub VersionButton_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles VersionButton.Click
		Dim message As New VB6.FixedLengthString(255)
		Dim nofChars As Short
		
        ' Get version information and display it in the version window
        ' If you get a run-time error here, check that you have copies
        ' of Argus.dll, icudt36.dll and icuuc36.dll in the bin folder.
        nofChars = icnArgusGetVersion(message.Value)

        VersionMessage.Text = VB.Left(message.Value, nofChars)
    End Sub

    Private Sub FolderBrowserDialog1_HelpRequest(sender As System.Object, e As System.EventArgs) Handles FolderBrowserDialog1.HelpRequest

    End Sub
End Class