/*	Simple example of using SDK to create a PDF processing application.
	A command line application like Argus.

	It demonstrates the use of the special VALUE macro, plus some very basic
	progress reporting callbacks.
*/

#include <stdio.h>

/*#include "ice.h"*/
#include "ipAbstract.h"
#include "argus_api.h"

static int userCancel(void* guiData);
static void fuelGauge(void* guiData, int percent);
static void msgOutput(void* guiData, int level, char* text);

/* -------------------------------------------------------------------------- */
int main(int argc, char** argv)
{
	/* Expects two arguments, PDF filename and output directory */
	char* pdfName;
	char* outDir;
	void* vargus;
	int ret;
	
	if (argc < 3)
	{
		fprintf(stderr, "usage: %s <pdf> <output path>\n", argv[0]);
		exit(1);
	}

	pdfName = argv[1];
	outDir = argv[2];

	/* Start up the SDK */
	vargus = icnArgusCreate();
	if (vargus == NULL)
	{
		fprintf(stderr, "%s: SDK Init failed: %s-%s\n", argv[0], 
					icnErrorGetStdText(), icnErrorGetText());
		exit(2);
	}

	/* This example always loads the same config */
	if (icnArgusLoadConfigs(vargus, "simple.cfg") != 0)
	{
		fprintf(stderr, "%s: %s-%s\n", argv[0], 
					icnErrorGetStdText(), icnErrorGetText());
		exit(3);
	}

	/* Change the config file so that output is put into the users requested
		folder. This is done by declaring a new VALUE key,value pair. This
		macro is used in simple.cfg
	*/
	if (icnArgusValueAdd(vargus, "userDir", outDir) != 0)
	{
		fprintf(stderr, "%s: %s-%s\n", argv[0], 
					icnErrorGetStdText(), icnErrorGetText());
	}

	/* Add some progress reporting functions */
	icnArgusSetProgress(vargus, msgOutput, fuelGauge, userCancel, NULL);

	/* set you passwords here if you need to
	ipSetUserPassword(myPassword);*/

	/* Process the file - all pages (page nos. 0, to -1 ) */
	ret = icnArgusExtract(vargus, argv+1, 1, 0, -1);

	/* Shut down the SDK */
	icnArgusDestroy(vargus);

	return(ret);
}

/* -------------------------------------------------------------------------- */
static void msgOutput(void* guiData, int level, char* text)
{
	/* Level determines the importance of the message */
	fprintf(stderr, "%2d. %s\n", level, text);
}

static void fuelGauge(void* guiData, int percent)
{
	fprintf(stderr, ".");
}

static int userCancel(void* guiData)
{
	/* Return true here to cause processing to stop */
	return(0);
}
